@@ -1,5 +1,7 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level
-#of this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from sql import Null
+
 from trytond.model import Workflow, ModelView, ModelSQL, fields
 from trytond.pyson import Not, Equal, Eval, Or, Bool
 from trytond import backend
@@ -115,7 +117,7 @@ class Inventory(Workflow, ModelSQL, ModelView):
     @classmethod
     @ModelView.button
     @Workflow.transition('done')
-    def confirm(self, inventories):
+    def confirm(cls, inventories):
         Move = Pool().get('stock.move')
         moves = []
         for inventory in inventories:
@@ -123,20 +125,20 @@ class Inventory(Workflow, ModelSQL, ModelView):
             for line in inventory.lines:
                 key = line.unique_key
                 if key in keys:
-                    self.raise_user_error('unique_line',
+                    cls.raise_user_error('unique_line',
                         (line.rec_name, inventory.rec_name))
                 keys.add(key)
                 move = line.get_move()
                 if move:
                     moves.append(move)
         if moves:
-            moves = Move.create([m._save_values for m in moves])
+            Move.save(moves)
             Move.do(moves)
 
     @classmethod
     @ModelView.button
     @Workflow.transition('cancel')
-    def cancel(self, inventories):
+    def cancel(cls, inventories):
         Line = Pool().get("stock.inventory.line")
         Line.cancel_move([l for i in inventories for l in i.lines])
 
@@ -273,7 +275,7 @@ class InventoryLine(ModelSQL, ModelView):
         # Migration from 3.0: use Move origin
         if table.column_exist('move'):
             cursor.execute(*sql_table.select(sql_table.id, sql_table.move,
-                    where=sql_table.move != None))
+                    where=sql_table.move != Null))
             for line_id, move_id in cursor.fetchall():
                 cursor.execute(*move_table.update(
                         columns=[move_table.origin],
@@ -291,13 +293,10 @@ class InventoryLine(ModelSQL, ModelView):
 
     @fields.depends('product')
     def on_change_product(self):
-        change = {}
-        change['unit_digits'] = 2
+        self.unit_digits = 2
         if self.product:
-            change['uom'] = self.product.default_uom.id
-            change['uom.rec_name'] = self.product.default_uom.rec_name
-            change['unit_digits'] = self.product.default_uom.digits
-        return change
+            self.uom = self.product.default_uom
+            self.unit_digits = self.product.default_uom.digits
 
     def get_rec_name(self, name):
         return self.product.rec_name
