@@ -1,10 +1,11 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import datetime
 import operator
 from decimal import Decimal
 from functools import partial
-from sql import Literal, Union, Column
+
+from sql import Literal, Union, Column, Null
 from sql.aggregate import Sum
 from sql.conditionals import Coalesce
 from sql.operators import Concat
@@ -16,6 +17,8 @@ from trytond.tools import reduce_ids
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
+from trytond.modules.product import price_digits
+
 __all__ = ['StockMixin', 'Move']
 
 STATES = {
@@ -194,6 +197,7 @@ class Move(Workflow, ModelSQL, ModelView):
             },
         depends=['state'])
     state = fields.Selection([
+        ('staging', 'Staging'),
         ('draft', 'Draft'),
         ('assigned', 'Assigned'),
         ('done', 'Done'),
@@ -208,14 +212,15 @@ class Move(Workflow, ModelSQL, ModelView):
                 Eval('context', {}).get('company', -1)),
             ],
         depends=['state'])
-    unit_price = fields.Numeric('Unit Price', digits=(16, 4),
+    unit_price = fields.Numeric('Unit Price', digits=price_digits,
         states={
             'invisible': Not(Bool(Eval('unit_price_required'))),
             'required': Bool(Eval('unit_price_required')),
             'readonly': Not(Equal(Eval('state'), 'draft')),
             },
         depends=['unit_price_required', 'state'])
-    cost_price = fields.Numeric('Cost Price', digits=(16, 4), readonly=True)
+    cost_price = fields.Numeric('Cost Price', digits=price_digits,
+        readonly=True)
     currency = fields.Many2One('currency.currency', 'Currency',
         states={
             'invisible': Not(Bool(Eval('unit_price_required'))),
@@ -223,7 +228,8 @@ class Move(Workflow, ModelSQL, ModelView):
             'readonly': Not(Equal(Eval('state'), 'draft')),
             },
         depends=['unit_price_required', 'state'])
-    unit_price_required = fields.Function(fields.Boolean('Unit Price Required'),
+    unit_price_required = fields.Function(
+        fields.Boolean('Unit Price Required'),
         'on_change_with_unit_price_required')
 
     @classmethod
@@ -264,6 +270,7 @@ class Move(Workflow, ModelSQL, ModelView):
             'no_origin': 'The stock move "%s" has no origin.',
             })
         cls._transitions |= set((
+                ('staging', 'draft'),
                 ('draft', 'assigned'),
                 ('draft', 'done'),
                 ('draft', 'cancel'),
@@ -345,7 +352,7 @@ class Move(Workflow, ModelSQL, ModelView):
                         columns=[sql_table.shipment],
                         values=[Concat(model + ',',
                                 Column(sql_table, column))],
-                        where=Column(sql_table, column) != None))
+                        where=Column(sql_table, column) != Null))
                 table.drop_column(column)
 
         # Add index on create_date
@@ -388,13 +395,10 @@ class Move(Workflow, ModelSQL, ModelView):
         Uom = pool.get('product.uom')
         Currency = pool.get('currency.currency')
 
-        res = {
-            'unit_price': Decimal('0.0'),
-            }
+        self.unit_price = Decimal('0.0')
         if self.product:
-            res['uom'] = self.product.default_uom.id
-            res['uom.rec_name'] = self.product.default_uom.rec_name
-            res['unit_digits'] = self.product.default_uom.digits
+            self.uom = self.product.default_uom
+            self.unit_digits = self.product.default_uom.digits
             unit_price = None
             if self.from_location and self.from_location.type in ('supplier',
                     'production'):
@@ -408,8 +412,7 @@ class Move(Workflow, ModelSQL, ModelView):
                 if self.currency and self.company:
                     unit_price = Currency.compute(self.company.currency,
                         unit_price, self.currency, round=False)
-                res['unit_price'] = unit_price
-        return res
+                self.unit_price = unit_price
 
     @fields.depends('product')
     def on_change_with_product_uom_category(self, name=None):
@@ -423,9 +426,7 @@ class Move(Workflow, ModelSQL, ModelView):
         Uom = pool.get('product.uom')
         Currency = pool.get('currency.currency')
 
-        res = {
-            'unit_price': Decimal('0.0'),
-            }
+        self.unit_price = Decimal('0.0')
         if self.product:
             if self.to_location and self.to_location.type == 'storage':
                 unit_price = self.product.cost_price
@@ -435,8 +436,7 @@ class Move(Workflow, ModelSQL, ModelView):
                 if self.currency and self.company:
                     unit_price = Currency.compute(self.company.currency,
                         unit_price, self.currency, round=False)
-                res['unit_price'] = unit_price
-        return res
+                self.unit_price = unit_price
 
     @fields.depends('from_location', 'to_location')
     def on_change_with_unit_price_required(self, name=None):
@@ -645,7 +645,8 @@ class Move(Workflow, ModelSQL, ModelView):
 
         vlist = [x.copy() for x in vlist]
         for vals in vlist:
-            assert vals.get('state', 'draft') == 'draft'
+            assert vals.get('state', cls.default_state()
+                ) in ['draft', 'staging']
 
             product = Product(vals['product'])
             uom = Uom(vals['uom'])
@@ -778,6 +779,8 @@ class Move(Workflow, ModelSQL, ModelView):
         success = True
         for move in moves:
             if move.state != 'draft':
+                if move.state == 'staging':
+                    success = False
                 continue
             to_location = move.to_location
             location_qties = {}
@@ -800,18 +803,19 @@ class Move(Workflow, ModelSQL, ModelView):
             for _, qty in to_pick:
                 picked_qties += qty
 
-            if picked_qties < move.quantity:
+            if move.quantity - picked_qties >= move.uom.rounding:
                 success = False
                 first = False
                 cls.write([move], {
-                    'quantity': move.quantity - picked_qties,
+                    'quantity': Uom.round(
+                            move.quantity - picked_qties, move.uom.rounding),
                     })
             else:
                 first = True
             for from_location, qty in to_pick:
                 values = {
                     'from_location': from_location.id,
-                    'quantity': qty,
+                    'quantity': Uom.round(qty, move.uom.rounding),
                     }
                 if first:
                     cls.write([move], values)
@@ -839,7 +843,8 @@ class Move(Workflow, ModelSQL, ModelView):
             stock_date_end: if set the date of the stock computation.
             stock_date_start: if set return the delta of the stock between the
                 two dates, (ignored if stock_date_end is missing).
-            stock_assign: if set compute also the assigned moves as done.
+            stock_assign: if set compute also the assigned outgoing moves as
+                done.
             forecast: if set compute the forecast quantity.
             stock_destinations: A list of location ids. If set, restrict the
                 computation to moves from and to those locations.
@@ -864,13 +869,8 @@ class Move(Workflow, ModelSQL, ModelView):
         Date = pool.get('ir.date')
         Period = pool.get('stock.period')
         Move = pool.get('stock.move')
-        Product = pool.get('product.product')
-        Template = pool.get('product.template')
 
         move = Move.__table__()
-        product = Product.__table__()
-        template = Template.__table__()
-
         today = Date.today()
 
         if not location_ids:
@@ -896,28 +896,30 @@ class Move(Workflow, ModelSQL, ModelView):
         if (context['stock_date_end'] < today
                 or (context['stock_date_end'] == today
                     and not context.get('forecast'))):
-            state_date_clause = (
+            state_date_clause = lambda stock_assign: (
                 move.state.in_(['done',
-                        context.get('stock_assign') and 'assigned' or 'done'])
+                        'assigned' if stock_assign else 'done'])
                 & (
                     (
-                        (move.effective_date == None)
+                        (move.effective_date == Null)
                         & (move.planned_date <= context['stock_date_end'])
                         )
                     | (move.effective_date <= context['stock_date_end'])
                     )
                 )
+            state_date_clause_in = state_date_clause(False)
+            state_date_clause_out = state_date_clause(
+                context.get('stock_assign'))
         # future date end: filter move on state done and date
         # before today, or on all state and date between today and
         # date_end.
         else:
-            state_date_clause = (
+            state_date_clause = lambda stock_assign: (
                 (move.state.in_(['done',
-                            context.get('stock_assign') and 'assigned'
-                            or 'done'])
+                            'assigned' if stock_assign else 'done'])
                     & (
                         (
-                            (move.effective_date == None)
+                            (move.effective_date == Null)
                             & (move.planned_date <= today)
                             )
                         | (move.effective_date <= today)
@@ -926,7 +928,7 @@ class Move(Workflow, ModelSQL, ModelView):
                 | (move.state.in_(['done', 'assigned', 'draft'])
                     & (
                         (
-                            (move.effective_date == None)
+                            (move.effective_date == Null)
                             & (Coalesce(move.planned_date, datetime.date.max)
                                 <= context['stock_date_end'])
                             & (Coalesce(move.planned_date, datetime.date.max)
@@ -939,33 +941,38 @@ class Move(Workflow, ModelSQL, ModelView):
                         )
                     )
                 )
+            state_date_clause_in = state_date_clause(False)
+            state_date_clause_out = state_date_clause(
+                context.get('stock_assign'))
 
         if context.get('stock_date_start'):
             if context['stock_date_start'] > today:
-                state_date_clause &= (
+                state_date_clause = lambda: (
                     move.state.in_(['done', 'assigned', 'draft'])
                     & (
                         (
-                            (move.effective_date == None)
+                            (move.effective_date == Null)
                             & (
                                 (move.planned_date >=
                                     context['stock_date_start'])
-                                | (move.planned_date == None)
+                                | (move.planned_date == Null)
                                 )
                             )
                         | (move.effective_date >= context['stock_date_start'])
                         )
                     )
+                state_date_clause_in &= state_date_clause()
+                state_date_clause_out &= state_date_clause()
             else:
-                state_date_clause &= (
+                state_date_clause = lambda stock_assign: (
                     (
                         move.state.in_(['done', 'assigned', 'draft'])
                         & (
                             (
-                                (move.effective_date == None)
+                                (move.effective_date == Null)
                                 & (
                                     (move.planned_date >= today)
-                                    | (move.planned_date == None)
+                                    | (move.planned_date == Null)
                                     )
                                 )
                             | (move.effective_date >= today)
@@ -973,18 +980,17 @@ class Move(Workflow, ModelSQL, ModelView):
                         )
                     | (
                         move.state.in_(['done',
-                                context.get('stock_assign') and 'assigned'
-                                or 'done'])
+                                'assigned' if stock_assign else 'done'])
                         & (
                             (
-                                (move.effective_date == None)
+                                (move.effective_date == Null)
                                 & (
                                     (
                                         (move.planned_date >=
                                             context['stock_date_start'])
                                         & (move.planned_date < today)
                                         )
-                                    | (move.planned_date == None)
+                                    | (move.planned_date == Null)
                                     )
                                 )
                             | (
@@ -995,6 +1001,9 @@ class Move(Workflow, ModelSQL, ModelView):
                             )
                         )
                     )
+                state_date_clause_in &= state_date_clause(False)
+                state_date_clause_out &= state_date_clause(
+                    context.get('stock_assign'))
         elif PeriodCache:
             periods = Period.search([
                     ('date', '<', context['stock_date_end']),
@@ -1002,9 +1011,11 @@ class Move(Workflow, ModelSQL, ModelView):
                     ], order=[('date', 'DESC')], limit=1)
             if periods:
                 period, = periods
-                state_date_clause &= (
+                state_date_clause = lambda: (
                     Coalesce(move.effective_date, move.planned_date,
                         datetime.date.max) > period.date)
+                state_date_clause_in &= state_date_clause()
+                state_date_clause_out &= state_date_clause()
 
         if with_childs:
             location_query = Location.search([
@@ -1013,11 +1024,10 @@ class Move(Workflow, ModelSQL, ModelView):
         else:
             location_query = location_ids[:]
 
-        from_ = move
         if PeriodCache:
             from_period = period_cache
+        where = where_period = Literal(True)
         if grouping_filter and any(grouping_filter):
-            where = where_period = Literal(True)
             for fieldname, grouping_ids in zip(grouping, grouping_filter):
                 if not grouping_ids:
                     continue
@@ -1032,16 +1042,6 @@ class Move(Workflow, ModelSQL, ModelView):
                     where &= column.in_(grouping_ids)
                     if PeriodCache:
                         where_period &= cache_column.in_(grouping_ids)
-        else:
-            where = where_period = template.active == True
-            from_ = from_.join(product, condition=move.product == product.id)
-            from_ = from_.join(template,
-                condition=product.template == template.id)
-            if PeriodCache:
-                from_period = from_period.join(product,
-                    condition=period_cache.product == product.id)
-                from_period = from_period.join(template,
-                    condition=product.template == template.id)
 
         if context.get('stock_destinations'):
             destinations = context['stock_destinations']
@@ -1059,21 +1059,22 @@ class Move(Workflow, ModelSQL, ModelView):
         # One that sums incoming moves towards locations, one that sums
         # outgoing moves and one for the period cache.  UNION ALL is used
         # because we already know that there will be no duplicates.
-        move_keys = [Column(move, key).as_(key) for key in grouping]
-        query = from_.select(move.to_location.as_('location'),
+        move_keys_alias = [Column(move, key).as_(key) for key in grouping]
+        move_keys = [Column(move, key) for key in grouping]
+        query = move.select(move.to_location.as_('location'),
             Sum(move.internal_quantity).as_('quantity'),
-            *move_keys,
-            where=state_date_clause
+            *move_keys_alias,
+            where=state_date_clause_in
             & where
             & move.to_location.in_(location_query)
             & (move.id.in_(move_rule_query) if move_rule_query
                 else Literal(True))
             & dest_clause_from,
             group_by=[move.to_location] + move_keys)
-        query = Union(query, from_.select(move.from_location.as_('location'),
+        query = Union(query, move.select(move.from_location.as_('location'),
                 (-Sum(move.internal_quantity)).as_('quantity'),
-                *move_keys,
-                where=state_date_clause
+                *move_keys_alias,
+                where=state_date_clause_out
                 & where
                 & move.from_location.in_(location_query)
                 & (move.id.in_(move_rule_query) if move_rule_query
