@@ -1,8 +1,8 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from itertools import chain
 from trytond.model import Workflow, ModelView, ModelSQL, fields
-from trytond.pyson import Equal, Eval, If, In, Get
+from trytond.pyson import Equal, Eval, If, In
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond.tools import grouped_slice
@@ -52,6 +52,10 @@ class Period(Workflow, ModelSQL, ModelView):
                     },
                 })
 
+    @staticmethod
+    def default_company():
+        return Transaction().context.get('company')
+
     @staticmethod
     def default_state():
         return 'draft'
