@@ -1,8 +1,9 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import datetime
 from decimal import Decimal
-from sql import Literal
+
+from sql import Literal, Null
 from sql.aggregate import Max
 from sql.functions import Now
 from sql.conditionals import Coalesce
@@ -107,7 +108,9 @@ class Product(object, StockMixin):
         context = {}
         trans_context = Transaction().context
         if trans_context.get('stock_date_end'):
-            context['_datetime'] = trans_context['stock_date_end']
+            # Use the last cost_price of the day
+            context['_datetime'] = datetime.datetime.combine(
+                trans_context['stock_date_end'], datetime.time.max)
         with Transaction().set_context(context):
             for product in products:
                 # The date could be before the product creation
@@ -261,7 +264,7 @@ class ProductQuantitiesByWarehouse(ModelSQL, ModelView):
             where=(move.product == product_id)
             & (move.from_location.in_(warehouse_query)
                 | move.to_location.in_(warehouse_query))
-            & (Coalesce(move.effective_date, move.planned_date) != None),
+            & (Coalesce(move.effective_date, move.planned_date) != Null),
             group_by=(date_column, move.product))
 
     @classmethod
