@@ -1,7 +1,8 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import operator
 import itertools
+import functools
 import datetime
 from sql import Table
 from sql.functions import Overlay, Position
@@ -12,7 +13,7 @@ from trytond.model import Workflow, ModelView, ModelSQL, fields
 from trytond.modules.company import CompanyReport
 from trytond.wizard import Wizard, StateTransition, StateView, Button
 from trytond import backend
-from trytond.pyson import Eval, Not, Equal, If, Or, And, Bool, In, Get, Id
+from trytond.pyson import Eval, Not, Equal, If, Or, And, Bool, In, Id
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.tools import reduce_ids, grouped_slice
@@ -245,6 +246,10 @@ class ShipmentIn(Workflow, ModelSQL, ModelView):
         table = TableHandler(cursor, cls, module_name)
         table.index_action('create_date', action='add')
 
+    @staticmethod
+    def default_planned_date():
+        return Pool().get('ir.date').today()
+
     @staticmethod
     def default_state():
         return 'draft'
@@ -262,10 +267,9 @@ class ShipmentIn(Workflow, ModelSQL, ModelView):
 
     @fields.depends('supplier')
     def on_change_supplier(self):
-        address = None
+        self.contact_address = None
         if self.supplier:
-            address = self.supplier.address_get()
-        return {'contact_address': address.id if address else None}
+            self.contact_address = self.supplier.address_get()
 
     @fields.depends('supplier')
     def on_change_with_supplier_location(self, name=None):
@@ -966,10 +970,9 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
 
     @fields.depends('customer')
     def on_change_customer(self):
-        address = None
+        self.delivery_address = None
         if self.customer:
-            address = self.customer.address_get(type='delivery')
-        return {'delivery_address': address.id if address else None}
+            self.delivery_address = self.customer.address_get(type='delivery')
 
     @fields.depends('customer')
     def on_change_with_customer_location(self, name=None):
@@ -1031,6 +1034,7 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
     def get_origins(self, name):
         return ', '.join(set(itertools.ifilter(None,
                     (m.origin_name for m in self.moves))))
+
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
@@ -1060,7 +1064,7 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
                     continue
                 to_create.append(shipment._get_inventory_move(move))
         if to_create:
-            Move.create([m._save_values for m in to_create])
+            Move.save(to_create)
 
     def _get_inventory_move(self, move):
         'Return inventory move for the outgoing move'
@@ -1077,12 +1081,13 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
             company=move.company,
             currency=move.currency,
             unit_price=move.unit_price,
+            state='staging' if move.state == 'staging' else 'draft',
             )
 
     @classmethod
     @Workflow.transition('assigned')
     def assign(cls, shipments):
-        pass
+        cls._sync_inventory_to_outgoing(shipments, create=True, write=False)
 
     @classmethod
     @ModelView.button
@@ -1112,7 +1117,7 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
             )
 
     @classmethod
-    def _sync_inventory_to_outgoing(cls, shipments):
+    def _sync_inventory_to_outgoing(cls, shipments, create=True, write=True):
         'Synchronise outgoing moves with inventory moves'
         pool = Pool()
         Move = pool.get('stock.move')
@@ -1150,15 +1155,17 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
                 else:
                     out_quantity = move.quantity
 
+                if not out_quantity:
+                    continue
                 unit_price = Uom.compute_price(move.product.default_uom,
                         move.product.list_price, move.uom)
                 to_create.append(shipment._get_outgoing_move(move))
                 to_create[-1].quantity = out_quantity
                 to_create[-1].unit_price = unit_price
-            if to_create:
-                Move.create([m._save_values for m in to_create])
+            if create and to_create:
+                Move.save(to_create)
 
-            #Re-read the shipment and remove exceeding quantities
+            # Re-read the shipment and remove exceeding quantities
             for move in shipment.outgoing_moves:
                 if move.state == 'cancel':
                     continue
@@ -1168,9 +1175,10 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
                     removed_qty = Uom.compute_qty(move.uom,
                         min(exc_qty, move.quantity), move.product.default_uom,
                         round=False)
-                    Move.write([move], {
-                            'quantity': max(0.0, move.quantity - exc_qty),
-                            })
+                    if write:
+                        Move.write([move], {
+                                'quantity': max(0.0, move.quantity - exc_qty),
+                                })
                     outgoing_qty[move.product.id] -= removed_qty
 
     @classmethod
@@ -1482,12 +1490,9 @@ class ShipmentOutReturn(Workflow, ModelSQL, ModelView):
 
     @fields.depends('customer')
     def on_change_customer(self):
-        address = None
+        self.delivery_address = None
         if self.customer:
-            address = self.customer.address_get(type='delivery')
-        return {
-            'delivery_address': address.id if address else None,
-            }
+            self.delivery_address = self.customer.address_get(type='delivery')
 
     @fields.depends('customer')
     def on_change_with_customer_location(self, name=None):
@@ -1715,7 +1720,7 @@ class AssignShipmentOut(Wizard):
                         'group_stock_force_assignment').in_(
                         Eval('context', {}).get('groups', [])),
                     }),
-            Button('Ok', 'end', 'tryton-ok', True),
+            Button('OK', 'end', 'tryton-ok', True),
             ])
     force = StateTransition()
 
@@ -2032,7 +2037,7 @@ class AssignShipmentInternal(Wizard):
                         'group_stock_force_assignment').in_(
                         Eval('context', {}).get('groups', [])),
                     }),
-            Button('Ok', 'end', 'tryton-ok', True),
+            Button('OK', 'end', 'tryton-ok', True),
             ])
     force = StateTransition()
 
@@ -2080,7 +2085,7 @@ class AssignShipmentInReturn(Wizard):
                         'group_stock_force_assignment').in_(
                         Eval('context', {}).get('groups', [])),
                 }),
-            Button('Ok', 'end', 'tryton-ok', True),
+            Button('OK', 'end', 'tryton-ok', True),
             ])
     force = StateTransition()
 
@@ -2105,11 +2110,11 @@ class DeliveryNote(CompanyReport):
     __name__ = 'stock.shipment.out.delivery_note'
 
     @classmethod
-    def parse(cls, report, objects, data, localcontext):
-        localcontext['product_name'] = lambda product_id, language: \
+    def get_context(cls, records, data):
+        report_context = super(DeliveryNote, cls).get_context(records, data)
+        report_context['product_name'] = lambda product_id, language: \
             cls.product_name(product_id, language)
-        return super(DeliveryNote, cls).parse(report, objects, data,
-                localcontext)
+        return report_context
 
     @classmethod
     def product_name(cls, product_id, language):
@@ -2118,189 +2123,61 @@ class DeliveryNote(CompanyReport):
             return Product(product_id).rec_name
 
 
-class PickingList(CompanyReport):
+class ShipmentReport(CompanyReport):
+    move_attribute = 'inventory_moves'
+
+    @classmethod
+    def get_context(cls, records, data):
+        report_context = super(ShipmentReport, cls).get_context(records, data)
+
+        compare_context = cls.get_compare_context(records, data)
+        sorted_moves = {}
+        for shipment in records:
+            sorted_moves[shipment.id] = sorted(
+                getattr(shipment, cls.move_attribute),
+                key=functools.partial(cls.get_compare_key, compare_context))
+        report_context['moves'] = sorted_moves
+
+        return report_context
+
+    @classmethod
+    def get_compare_context(cls, objects, data):
+        from_location_ids = set()
+        to_location_ids = set()
+        for obj in objects:
+            for move in getattr(obj, cls.move_attribute):
+                from_location_ids.add(move.from_location.id)
+                to_location_ids.add(move.to_location.id)
+
+        return {
+            'from_location_ids': list(from_location_ids),
+            'to_location_ids': list(to_location_ids),
+            }
+
+    @staticmethod
+    def get_compare_key(compare_context, move):
+        from_location_ids = compare_context['from_location_ids']
+        to_location_ids = compare_context['to_location_ids']
+        return [from_location_ids.index(move.from_location.id),
+                to_location_ids.index(move.to_location.id)]
+
+
+class PickingList(ShipmentReport):
     'Picking List'
     __name__ = 'stock.shipment.out.picking_list'
 
-    @classmethod
-    def parse(cls, report, objects, data, localcontext):
-        compare_context = cls.get_compare_context(report, objects, data)
 
-        sorted_moves = {}
-        for shipment in objects:
-            sorted_moves[shipment.id] = sorted(
-                shipment.inventory_moves,
-                lambda x, y: cmp(cls.get_compare_key(x, compare_context),
-                    cls.get_compare_key(y, compare_context))
-                )
-
-        localcontext['moves'] = sorted_moves
-
-        return super(PickingList, cls).parse(report, objects, data,
-            localcontext)
-
-    @staticmethod
-    def get_compare_context(report, objects, data):
-        Location = Pool().get('stock.location')
-        from_location_ids = set()
-        to_location_ids = set()
-        for obj in objects:
-            for move in obj.inventory_moves:
-                from_location_ids.add(move.from_location.id)
-                to_location_ids.add(move.to_location.id)
-
-        from_locations = Location.browse(list(from_location_ids))
-        to_locations = Location.browse(list(to_location_ids))
-
-        return {
-            'from_location_ids': [l.id for l in from_locations],
-            'to_location_ids': [l.id for l in to_locations],
-            }
-
-    @staticmethod
-    def get_compare_key(move, compare_context):
-        from_location_ids = compare_context['from_location_ids']
-        to_location_ids = compare_context['to_location_ids']
-        return [from_location_ids.index(move.from_location.id),
-                to_location_ids.index(move.to_location.id)]
-
-
-class SupplierRestockingList(CompanyReport):
+class SupplierRestockingList(ShipmentReport):
     'Supplier Restocking List'
     __name__ = 'stock.shipment.in.restocking_list'
 
-    @classmethod
-    def parse(cls, report, objects, data, localcontext):
-        compare_context = cls.get_compare_context(report, objects, data)
 
-        sorted_moves = {}
-        for shipment in objects:
-            sorted_moves[shipment.id] = sorted(
-                shipment.inventory_moves,
-                lambda x, y: cmp(cls.get_compare_key(x, compare_context),
-                    cls.get_compare_key(y, compare_context))
-                )
-
-        localcontext['moves'] = sorted_moves
-
-        return super(SupplierRestockingList, cls).parse(report, objects,
-                data, localcontext)
-
-    @staticmethod
-    def get_compare_context(report, objects, data):
-        Location = Pool().get('stock.location')
-        from_location_ids = set()
-        to_location_ids = set()
-        for obj in objects:
-            for move in obj.inventory_moves:
-                from_location_ids.add(move.from_location.id)
-                to_location_ids.add(move.to_location.id)
-
-        from_locations = Location.browse(list(from_location_ids))
-        to_locations = Location.browse(list(to_location_ids))
-
-        return {
-            'from_location_ids': [l.id for l in from_locations],
-            'to_location_ids': [l.id for l in to_locations],
-            }
-
-    @staticmethod
-    def get_compare_key(move, compare_context):
-        from_location_ids = compare_context['from_location_ids']
-        to_location_ids = compare_context['to_location_ids']
-        return [from_location_ids.index(move.from_location.id),
-                to_location_ids.index(move.to_location.id)]
-
-
-class CustomerReturnRestockingList(CompanyReport):
+class CustomerReturnRestockingList(ShipmentReport):
     'Customer Return Restocking List'
     __name__ = 'stock.shipment.out.return.restocking_list'
 
-    @classmethod
-    def parse(cls, report, objects, data, localcontext):
-        compare_context = cls.get_compare_context(report, objects, data)
 
-        sorted_moves = {}
-        for shipment in objects:
-            sorted_moves[shipment.id] = sorted(
-                shipment.inventory_moves,
-                lambda x, y: cmp(cls.get_compare_key(x, compare_context),
-                    cls.get_compare_key(y, compare_context))
-                )
-
-        localcontext['moves'] = sorted_moves
-
-        return super(CustomerReturnRestockingList, cls).parse(report,
-                objects, data, localcontext)
-
-    @staticmethod
-    def get_compare_context(report, objects, data):
-        Location = Pool().get('stock.location')
-        from_location_ids = set()
-        to_location_ids = set()
-        for obj in objects:
-            for move in obj.inventory_moves:
-                from_location_ids.add(move.from_location.id)
-                to_location_ids.add(move.to_location.id)
-
-        from_locations = Location.browse(list(from_location_ids))
-        to_locations = Location.browse(list(to_location_ids))
-
-        return {
-            'from_location_ids': [l.id for l in from_locations],
-            'to_location_ids': [l.id for l in to_locations],
-            }
-
-    @staticmethod
-    def get_compare_key(move, compare_context):
-        from_location_ids = compare_context['from_location_ids']
-        to_location_ids = compare_context['to_location_ids']
-        return [from_location_ids.index(move.from_location.id),
-                to_location_ids.index(move.to_location.id)]
-
-
-class InteralShipmentReport(CompanyReport):
+class InteralShipmentReport(ShipmentReport):
     'Interal Shipment Report'
     __name__ = 'stock.shipment.internal.report'
-
-    @classmethod
-    def parse(cls, report, objects, data, localcontext=None):
-        compare_context = cls.get_compare_context(report, objects, data)
-
-        sorted_moves = {}
-        for shipment in objects:
-            sorted_moves[shipment.id] = sorted(
-                shipment.moves,
-                lambda x, y: cmp(cls.get_compare_key(x, compare_context),
-                    cls.get_compare_key(y, compare_context))
-                )
-
-        localcontext['moves'] = sorted_moves
-
-        return super(InteralShipmentReport, cls).parse(report, objects,
-            data, localcontext)
-
-    @staticmethod
-    def get_compare_context(report, objects, data):
-        Location = Pool().get('stock.location')
-        from_location_ids = set()
-        to_location_ids = set()
-        for obj in objects:
-            for move in obj.moves:
-                from_location_ids.add(move.from_location.id)
-                to_location_ids.add(move.to_location.id)
-
-        from_locations = Location.browse(list(from_location_ids))
-        to_locations = Location.browse(list(to_location_ids))
-
-        return {
-            'from_location_ids': [l.id for l in from_locations],
-            'to_location_ids': [l.id for l in to_locations],
-            }
-
-    @staticmethod
-    def get_compare_key(move, compare_context):
-        from_location_ids = compare_context['from_location_ids']
-        to_location_ids = compare_context['to_location_ids']
-        return [from_location_ids.index(move.from_location.id),
-                to_location_ids.index(move.to_location.id)]
+    move_attribute = 'moves'
